-- babel
babel.init({ locale = "en-UK", locales_folders = { "resources/translations" } })

-- lurker
local lurker_scan_key = " "

-- controllers
Controllers.init({
    menu_up = {type="button", default={'kb_up','gpb_dpup','gpa_lefty_-'}},
    menu_down = {type="button", default={'kb_down','gpb_dpdown','gpa_lefty_+'}},
    menu_select = {type="button", default={'kb_return','gpb_a'}},
    menu_back = {type="button", default={'kb_escape','gpb_b'}},
    lurker_update = {type="button", default={'kb_'..lurker_scan_key}},
    toggle_fps = {type="button", default={'kb_f'}},
    toggle_profiler = {type="button", default={'kb_p'}},
    oak1 = {type="button", default={'kb_1'}},
    oak2 ={type="button", default={'kb_2'}},
    oak3 = {type="button", default={'kb_3'}},
})
Controllers.setPlayer('player_1','keyboard',1)

-- loveplot
FPSplot = loveplot.new(100)
FPSplot.title = "Frames Per Second"
FPSplot.upperLimit = 30
FPSplot:setValueLimits(0,FPSplot.upperLimit)

function FPSplot.update(enabled, val)
	if enabled then
		FPSplot:nextValue(val)
		local max = FPSplot.upperLimit
		for key,val in pairs(FPSplot.values) do
			max = math.max(max,val)
		end
		FPSplot:setValueLimits(0,10*math.ceil(max/10))
	end
end

-- demo stuff
foundation.showFPSplot = false
foundation.showProfilers = false

tree = {}
tree.new = function(values)

	local t = {}
	t.x = values.x
	t.y = values.y
	t.angle = values.angle
	t.scale = values.scale
	t.water = values.water
	t.nutrients = values.nutrients
	t.branches = {}
	t.branches[1] = {parent = false, offset = false, angle = 0, length = 0, x1 = values.x, y1 = values.y, growing = true}
	t.visible = true

	function t:newBranch(parentID, offsetPercent)
		local parent = self.branches[parentID]
		self.branches[#self.branches+1] = {
		parent = parentID,
		offset = parent.length*offsetPercent/100,
		angle = parent.angle + (love.math.random()-0.5),
		length = 0,
		x1 = parent.x2,
		y1 = parent.y2,
		x2 = parent.x2,
		y2 = parent.y2,
		growing = true
		}
	end

	function t:newJunction(parentID)
		local parent = self.branches[parentID]
		local angle = love.math.random()-0.5
		self.branches[#self.branches+1] = {
		parent = parentID,
		offset = parent.length,
		angle = parent.angle + angle,
		length = 0,
		x1 = parent.x2,
		y1 = parent.y2,
		x2 = parent.x2,
		y2 = parent.y2,
		growing = true
		}
		self.branches[#self.branches+1] = {
		parent = parentID,
		offset = parent.length,
		angle = parent.angle - angle,
		length = 0,
		x1 = parent.x2,
		y1 = parent.y2,
		x2 = parent.x2,
		y2 = parent.y2,
		growing = true
		}
		self.branches[parentID].growing = false
	end


	function t:returnPoints(branch)
		return branch.x1, branch.y1, branch.x2, branch.y2
	end

	function t:update(dt)
		for id,branch in pairs(self.branches) do
			if id == 1 then
				if love.math.random() > 1 - branch.length/(50000 * (50/#self.branches)) then
					self.branches[id].growing = false
				end
			end
			if branch.growing then
				self.branches[id].length = self.water*self.nutrients*dt + branch.length
				--print("length of "..id..":", self.branches[id].length)
				self.branches[id].x2, self.branches[id].y2 = branch.x1 + self.branches[id].length*math.sin(branch.angle+math.pi), branch.y1 + self.branches[id].length*math.cos(branch.angle+math.pi)
				if love.math.random() > 0.995 then
					if love.math.random() < 0.8 then
						t:newBranch(id,love.math.random(10,100))
					else
						t:newJunction(id)
					end
				end
				if love.math.random() > 1 - branch.length/(50000 * (50/id)) then
					self.branches[id].growing = false
				end
			end
		end
		if not self.age then self.age = dt else self.age = dt+self.age end
	end

	function t:draw()
		if self.visible then
			love.graphics.setColor({255, 255, 255, 255})
			for id,branch in pairs(self.branches) do
				if branch.parent then
					love.graphics.setLineWidth( math.ceil(self.scale/(branch.parent+1)) )
				else
					love.graphics.setLineWidth( self.scale )
				end
				love.graphics.line(self:returnPoints(self.branches[id]))
			end
		end
	end

	function t:toggle()
		self.visible = not self.visible
	end
	return t
end

oak1 = tree.new({
	x = 200,
	y = 500,
	angle = math.pi/4,
	scale = 8,
	water = 5,
	nutrients = 2,
	})

oak2 = tree.new({
	x = 400,
	y = 500,
	angle = math.pi/4,
	scale = 8,
	water = 5,
	nutrients = 2,
	})

oak3 = tree.new({
	x = 600,
	y = 500,
	angle = math.pi/4,
	scale = 8,
	water = 5,
	nutrients = 2,
	})

updateProfiler:hook(FPSplot, 'nextValue')